# -*- coding: UTF-8 -*-
# Author: Fabian Steiner, <info@fabis-site.net>

import sys
import re
from string import Template
from multiseat.constants import *

class TemplateParserException(Exception):
    def __init__(self, msg):
        self.msg = msg

class Seat(object):
    KNOWN_DIRECTIVES = ['keyboard_identifier', 
                                         'keyboard_phys',
                                         'keyboard_device',
                                         'mouse_identifier',
                                         'mouse_phys',
                                         'mouse_device',
                                         'video_card_identifier',
                                         'video_card_busid',
                                         'screen_number',
                                         'monitor_identifier',
                                         'screen_identifier',
                                         'layout_identifier']
    
    def __init__(self):
        for directive in Seat.KNOWN_DIRECTIVES:
            setattr(self, directive, None)
    
    def _prepare(self):
        try:
            f = open(SEAT_TEMPLATE)
        except IOError, errstr:
            raise TemplateParserException, 'Can not open %s for reading. Error: %s' % (SEAT_TEMPLATE, errstr)
        self.template = f.read()
        f.close()
        directives = re.compile(r'\$[A-Z_]+').findall(self.template)
        if not directives:
            raise TemplateParserException, 'Can not find any configuration directives in template.'
        self.directives = self._modifyDirectives(directives, 'lower')
        unknown_directives = set(self.directives) - set(Seat.KNOWN_DIRECTIVES)
        if unknown_directives:
            raise TemplateParserException, 'Found unknown directive(s): %s.' % ' '.join(unknown_directives)
        del directives, unknown_directives
        for directive in self.directives:
            if getattr(self, directive) is None:
                raise TemplateParserException, 'Directive %s has got no value.' % directive
        
    def _modifyDirectives(self, directives, action='lower'):
            if action == 'upper':
                return map(lambda x: x.upper(), directives)
            elif action == 'lower':
                return map(lambda x: x[1:].lower(), directives)
    
    def _toDict(self):
        return dict(zip(self._modifyDirectives(self.directives, 'upper'), \
                                map(lambda directive: getattr(self, directive), self.directives)))
    
    def substitute(self):
        self._prepare()
        return Template(self.template).substitute(self._toDict())
